###########################################################################################################
#
#  default
#  
#  @author simon jokuschies
#  @version 2.0
#  @contact info@leafpictures.de
#  @website www.leafpictures.de
#
#  description:
#  Set knob defaults on the fly so you dont't have to write them manually into your nuke home directory.
#  Just set a knob like you want to have it as knobdefault and then right click, choose myDefaults - set as knob default
#  If you want to get rid of the knob default just right click on the knob and choose myDefaults - reset.
#
#  versions
#  v2.0:
#  right click a knob and choose default-> show knob list. A pop up window appears showing all knobs of the 
#  node. Choose a knob and set its new default value. With that you get access to set default values of all knobs
#  of each node, even if it's a dropdown (didn't work in previous versions, now you can do that.)
#
#  instalation
#
#  put myDefaults folder inside nuke home directory. In your int.py write (without the '#'):
#  nuke.pluginAddPath("default")
#
##########################################################################################################

# no need to change anything from here. Edit only if you exactly know what you're doing.

import nuke
import os
import helper
import knobDefaults

default_dir = os.path.dirname(__file__)
knobInit = default_dir+"/knobDefaults.py"
ctrl = False
knob_list_dont_include = ["xpos", "ypos", "name"]


def create_default():
    """
    create custom knobDefault value
    """

    n = nuke.thisNode()
    k = nuke.thisKnob()

    # set default for current nuke session
    nuke.knobDefault("{node}.{knob}".format(node=n.Class(), knob=k.name()), "{val}".format(val=k.value()))

    # set
    update_knob_init(n.Class(), k.name(), k.value(), "write")


def reset_to_default():
    """
    reset to standard knob default value
    """

    n = nuke.thisNode()
    k = nuke.thisKnob()

    k.setValue(k.defaultValue())
    nuke.knobDefault("{node}.{knob}".format(node=n.Class(), knob=k.name()), "{val}".format(val=k.defaultValue()))
    update_knob_init(n.Class(), k.name(), "", "del")


def update_knob_init(node, knob, value, mode):
    """
    update knob knob_defaults.py - delete or append knobDefault depending on mode
    """

    knob_defaults = helper.open_file_return_arr(knobInit)

    if mode == "del":

        found = 0

        for d in knob_defaults:
            if "{node}.{knob}".format(node=node, knob=knob) in d:
                found += 1
                if ctrl:
                    print "found in knobInit"
                    print d
                knob_defaults.remove(d)
            else:
                pass
        if found < 1:
            if ctrl:
                print "not found in knobInit"

        # write new knobInit
        try:
            f = open(knobInit, 'w+')
            for d in knob_defaults:
                f.write(d + "\n")
            f.close()
        except IOError:
            nuke.message("an error occurred while trying to edit the knob_defaults file")

    if mode == "write":
        # get rid of old knob_defaults of the current knob and write new knobDefault
        update_knob_init(node, knob, "", "del")
        try:
            f = open(knobInit, 'a')
            new_knob_default = 'nuke.knobDefault("{node}.{knob}", "{val}")\n'.format(node=node, knob=knob, val=value)
            f.write(new_knob_default)
            f.close()
        except IOError:
            nuke.message("an error occurred while trying to edit the knob_defaults file")


def show_knob_list():
    """
    show window for all knobs setting individual knob defaults
    :return: None
    """

    try:
        sel = nuke.selectedNode()
    except ValueError:
        return None

    dict_all_knobs = sel.knobs()
    list_knobs = ["-----"]
    for name in dict_all_knobs.keys():
        if name not in knob_list_dont_include:
            list_knobs.append(name)

    list_knobs.sort()

    p = nuke.Panel("all knobs")
    p.addEnumerationPulldown("knob", " ".join(list_knobs))
    p.addSingleLineInput("new default value", "")
    if p.show():
        # set default for current nuke session
        nuke.knobDefault("{node}.{knob}".format(node=sel.Class(), knob=p.value("knob")),
                         "{val}".format(val=p.value("new default value")))

        # set
        update_knob_init(sel.Class(), p.value("knob"), p.value("new default value"), "write")





